@@ -21,18 +21,22 @@ VALID_LEN_EXPRESSION = [5, 6]
 
 
 class CroniterError(ValueError):
+    """ General top-level Croniter base exception """
     pass
 
 
 class CroniterBadCronError(CroniterError):
+    """ Syntax, unknown value, or range error within a cron expression """
     pass
 
 
 class CroniterBadDateError(CroniterError):
+    """ Unable to find next/prev timestamp match """
     pass
 
 
-class CroniterNotAlphaError(CroniterError):
+class CroniterNotAlphaError(CroniterBadCronError):
+    """ Cron syntax contains an invalid day or month abreviation """
     pass
 
 
@@ -51,12 +55,15 @@ class croniter(object):
     )
 
     ALPHACONV = (
-        {},
-        {},
-        {"l": "l"},
+        {},  # 0: min
+        {},  # 1: hour
+        {"l": "l"},  # 2: dom
+        # 3: mon
         {'jan': 1, 'feb': 2, 'mar': 3, 'apr': 4, 'may': 5, 'jun': 6,
          'jul': 7, 'aug': 8, 'sep': 9, 'oct': 10, 'nov': 11, 'dec': 12},
+        # 4: dow
         {'sun': 0, 'mon': 1, 'tue': 2, 'wed': 3, 'thu': 4, 'fri': 5, 'sat': 6},
+        # command/user
         {}
     )
 
@@ -69,14 +76,29 @@ class croniter(object):
         {},
     )
 
+    LEN_MEANS_ALL = (
+        60,
+        24,
+        31,
+        12,
+        7,
+        60
+    )
+
     bad_length = 'Exactly 5 or 6 columns has to be specified for iterator' \
                  'expression.'
 
     def __init__(self, expr_format, start_time=None, ret_type=float,
-                 day_or=True):
+                 day_or=True, max_years_between_matches=None, is_prev=False):
         self._ret_type = ret_type
         self._day_or = day_or
 
+        self._max_years_btw_matches_explicitly_set = (
+            max_years_between_matches is not None)
+        if not self._max_years_btw_matches_explicitly_set:
+            max_years_between_matches = 50
+        self._max_years_between_matches = max(int(max_years_between_matches), 1)
+
         if start_time is None:
             start_time = time()
 
@@ -88,6 +110,7 @@ class croniter(object):
         self.set_current(start_time)
 
         self.expanded, self.nth_weekday_of_month = self.expand(expr_format)
+        self._is_prev = is_prev
 
     @classmethod
     def _alphaconv(cls, index, key, expressions):
@@ -152,30 +175,10 @@ class croniter(object):
         return (td.microseconds + (td.seconds + td.days * 24 * 3600) * 10**6) \
             / 10**6
 
-    # iterator protocol, to enable direct use of croniter
-    # objects in a loop, like "for dt in croniter('5 0 * * *'): ..."
-    # or for combining multiple croniters into single
-    # dates feed using 'itertools' module
-    def __iter__(self):
-        return self
-    __next__ = next = get_next
-
-    def all_next(self, ret_type=None):
-        '''Generator of all consecutive dates. Can be used instead of
-        implicit call to __iter__, whenever non-default
-        'ret_type' has to be specified.
-        '''
-        while True:
-            yield self._get_next(ret_type or self._ret_type, is_prev=False)
-
-    def all_prev(self, ret_type=None):
-        '''Generator of all previous dates.'''
-        while True:
-            yield self._get_next(ret_type or self._ret_type, is_prev=True)
-
-    iter = all_next  # alias, you can call .iter() instead of .all_next()
-
-    def _get_next(self, ret_type=None, is_prev=False):
+    def _get_next(self, ret_type=None, is_prev=None):
+        if is_prev is None:
+            is_prev = self._is_prev
+        self._is_prev = is_prev
         expanded = self.expanded[:]
         nth_weekday_of_month = self.nth_weekday_of_month.copy()
 
@@ -233,6 +236,45 @@ class croniter(object):
             result = dtresult
         return result
 
+    # iterator protocol, to enable direct use of croniter
+    # objects in a loop, like "for dt in croniter('5 0 * * *'): ..."
+    # or for combining multiple croniters into single
+    # dates feed using 'itertools' module
+    def all_next(self, ret_type=None):
+        '''Generator of all consecutive dates. Can be used instead of
+        implicit call to __iter__, whenever non-default
+        'ret_type' has to be specified.
+        '''
+        # In a Python 3.7+ world:  contextlib.supress and contextlib.nullcontext could be used instead
+        try:
+            while True:
+                self._is_prev = False
+                yield self._get_next(ret_type or self._ret_type)
+        except CroniterBadDateError:
+            if self._max_years_btw_matches_explicitly_set:
+                return
+            else:
+                raise
+
+    def all_prev(self, ret_type=None):
+        '''Generator of all previous dates.'''
+        try:
+            while True:
+                self._is_prev = True
+                yield self._get_next(ret_type or self._ret_type)
+        except CroniterBadDateError:
+            if self._max_years_btw_matches_explicitly_set:
+                return
+            else:
+                raise
+
+    def iter(self, *args, **kwargs):
+        return (self._is_prev and self.all_prev or self.all_next)
+
+    def __iter__(self):
+        return self
+    __next__ = next = _get_next
+
     def _calc(self, now, expanded, nth_weekday_of_month, is_prev):
         if is_prev:
             now = math.ceil(now)
@@ -405,7 +447,7 @@ class croniter(object):
                  proc_minute,
                  proc_second]
 
-        while abs(year - current_year) <= 1:
+        while abs(year - current_year) <= self._max_years_between_matches:
             next = False
             for proc in procs:
                 (changed, dst) = proc(dst)
@@ -493,7 +535,7 @@ class croniter(object):
                 if i == 4:
                     e, sep, nth = str(e).partition('#')
                     if nth and not re.match(r'[1-5]', nth):
-                        raise CroniterBadDateError(
+                        raise CroniterBadCronError(
                             "[{0}] is not acceptable".format(expr_format))
 
                 t = re.sub(r'^\*(\/.+)$', r'%d-%d\1' % (
@@ -510,6 +552,8 @@ class croniter(object):
 
                 if m:
                     (low, high, step) = m.group(1), m.group(2), m.group(4) or 1
+                    if i == 2 and high == 'l':
+                        high = '31'
 
                     if not any_int_re.search(low):
                         low = "{0}".format(cls._alphaconv(i, low, expressions))
@@ -525,7 +569,7 @@ class croniter(object):
                             # handle -Sun notation -> 7
                             high = '7'
                         else:
-                            raise CroniterBadDateError(
+                            raise CroniterBadCronError(
                                 "[{0}] is not acceptable".format(expr_format))
 
                     low, high, step = map(int, [low, high, step])
@@ -569,7 +613,11 @@ class croniter(object):
                             nth_weekday_of_month[t] = set()
                         nth_weekday_of_month[t].add(int(nth))
 
+            res = set(res)
             res = natsort.natsorted(res)
+            if len(res) == cls.LEN_MEANS_ALL[i]:
+                res = ['*']
+
             expanded.append(['*'] if (len(res) == 1
                                       and res[0] == '*')
                             else res)
@@ -589,6 +637,59 @@ class croniter(object):
     def match(cls, cron_expression, testdate, day_or=True):
         cron = cls(cron_expression, testdate, ret_type=datetime.datetime, day_or=day_or)
         td, ms1 = cron.get_current(datetime.datetime), relativedelta(microseconds=1)
-        cron.set_current(td + ms1)
+        if not td.microsecond:
+            td = td + ms1
+        cron.set_current(td)
         tdp, tdt = cron.get_current(), cron.get_prev()
         return (max(tdp, tdt) - min(tdp, tdt)).total_seconds() < 60
+
+
+def croniter_range(start, stop, expr_format, ret_type=None, day_or=True, exclude_ends=False):
+    """
+    Generator that provides all times from start to stop matching the given cron expression.
+    If the cron expression matches either 'start' and/or 'stop', those times will be returned as
+    well unless 'exclude_ends=True' is passed.
+
+    You can think of this function as sibling to the builtin range function for datetime objects.
+    Like range(start,stop,step), except that here 'step' is a cron expression.
+    """
+    auto_rt = datetime.datetime
+    if type(start) != type(stop):
+        raise TypeError("The start and stop must be same type.  {0} != {1}".
+                        format(type(start), type(stop)))
+    if isinstance(start, (float, int)):
+        start, stop = (datetime.datetime.utcfromtimestamp(t) for t in (start, stop))
+        auto_rt = float
+    if ret_type is None:
+        ret_type = auto_rt
+    if not exclude_ends:
+        ms1 = relativedelta(microseconds=1)
+        if start < stop:    # Forward (normal) time order
+            start -= ms1
+            stop += ms1
+        else:               # Reverse time order
+            start += ms1
+            stop -= ms1
+    year_span = math.floor(abs(stop.year - start.year)) + 1
+    ic = croniter(expr_format, start, ret_type=datetime.datetime, day_or=day_or,
+                  max_years_between_matches=year_span)
+    # define a continue (cont) condition function and step function for the main while loop
+    if start < stop:        # Forward
+        def cont(v):
+            return v < stop
+        step = ic.get_next
+    else:                   # Reverse
+        def cont(v):
+            return v > stop
+        step = ic.get_prev
+    try:
+        dt = step()
+        while cont(dt):
+            if ret_type is float:
+                yield ic.get_current(float)
+            else:
+                yield dt
+            dt = step()
+    except CroniterBadDateError:
+        # Stop iteration when this exception is raised; no match found within the given year range
+        return
