@@ -6,7 +6,8 @@ from datetime import datetime, timedelta
 from functools import partial
 from time import sleep
 import pytz
-from croniter import croniter, CroniterBadDateError, CroniterBadCronError, CroniterNotAlphaError
+import croniter as cr
+from croniter import croniter, croniter_range, CroniterBadDateError, CroniterBadCronError, CroniterNotAlphaError
 from croniter.tests import base
 from tzlocal import get_localzone
 import dateutil.tz
@@ -299,6 +300,32 @@ class CroniterTest(base.TestCase):
         self.assertEqual(22, next.hour)
         self.assertEqual(30, next.minute)
 
+    def testOptimizeCronExpressions(self):
+        """ Non-optimal cron expressions that can be simplified."""
+        wildcard = ['*']
+        m, h, d, mon, dow, s = range(6)
+        # Test each field individually
+        self.assertEqual(croniter('0-59 0 0 0 0').expanded[m], wildcard)
+        self.assertEqual(croniter('0 0-23 0 0 0').expanded[h], wildcard)
+        self.assertEqual(croniter('0 0 0-31 0 0').expanded[d], wildcard)
+        self.assertEqual(croniter('0 0 0 1-12 0').expanded[mon], wildcard)
+        self.assertEqual(croniter('0 0 0 0 0-6').expanded[dow], wildcard)
+        self.assertEqual(croniter('0 0 0 0 1-7').expanded[dow], wildcard)
+        self.assertEqual(croniter('0 0 0 0 0 0-59').expanded[s], wildcard)
+        # Real life examples
+        self.assertEqual(croniter('30 1-12,0,10-23 15-21 * fri').expanded[h], wildcard)
+        self.assertEqual(croniter('30 1-23,0 15-21 * fri').expanded[h], wildcard)
+
+    def testBlockDupRanges(self):
+        """ Ensure that duplicate/overlapping ranges are squashed """
+        m, h, d, mon, dow, s = range(6)
+        self.assertEqual(croniter('* 5,5,1-6 * * *').expanded[h], [1,2,3,4,5,6])
+        self.assertEqual(croniter('* * * * 2-3,4-5,3,3,3').expanded[dow], [2,3,4,5])
+        self.assertEqual(croniter('* * * * * 1,5,*/20,20,15').expanded[s], [0, 1, 5, 15, 20, 40])
+        self.assertEqual(croniter('* 4,1-4,5,4 * * *').expanded[h], [1, 2, 3, 4, 5])
+        # Real life example
+        self.assertEqual(croniter('59 23 * 1 wed,fri,mon-thu,tue,tue').expanded[dow], [1,2,3,4,5])
+
     def testPrevMinute(self):
         base = datetime(2010, 8, 25, 15, 56)
         itr = croniter('*/1 * * * *', base)
@@ -1063,5 +1090,206 @@ class CroniterTest(base.TestCase):
             '2020-03-29T02:01:00+02:00',
             '2020-03-29T03:01:00+02:00'])
 
+
+class CroniterRangeTest(base.TestCase):
+
+    def test_1day_step(self):
+        start = datetime(2016, 12, 2)
+        stop = datetime(2016, 12, 10)
+        fwd = list(croniter_range(start, stop, '0 0 * * *'))
+        self.assertEqual(len(fwd), 9)
+        self.assertEqual(fwd[0], start)
+        self.assertEqual(fwd[-1], stop)
+        # Test the same, but in reverse
+        rev = list(croniter_range(stop, start, '0 0 * * *'))
+        self.assertEqual(len(rev), 9)
+        # Ensure forward/reverse are a mirror image
+        rev.reverse()
+        self.assertEqual(fwd, rev)
+
+    def test_1day_step_no_ends(self):
+        # Test without ends (exclusive)
+        start = datetime(2016, 12, 2)
+        stop = datetime(2016, 12, 10)
+        fwd = list(croniter_range(start, stop, '0 0 * * *', exclude_ends=True))
+        self.assertEqual(len(fwd), 7)
+        self.assertNotEqual(fwd[0], start)
+        self.assertNotEqual(fwd[-1], stop)
+        # Test the same, but in reverse
+        rev = list(croniter_range(stop, start, '0 0 * * *', exclude_ends=True))
+        self.assertEqual(len(rev), 7)
+        self.assertNotEqual(fwd[0], stop)
+        self.assertNotEqual(fwd[-1], start)
+
+    def test_1month_step(self):
+        start = datetime(1982, 1, 1)
+        stop = datetime(1983, 12, 31)
+        res = list(croniter_range(start, stop, '0 0 1 * *'))
+        self.assertEqual(len(res), 24)
+        self.assertEqual(res[0], start)
+        self.assertEqual(res[5].day, 1)
+        self.assertEqual(res[-1], datetime(1983, 12, 1))
+
+    def test_1minute_step_float(self):
+        start = datetime(2000, 1, 1, 0, 0)
+        stop =  datetime(2000, 1, 1, 0, 1)
+        res = list(croniter_range(start, stop, '* * * * *', ret_type=float))
+        self.assertEqual(len(res), 2)
+        self.assertEqual(res[0], 946684800.0)
+        self.assertEqual(res[-1] - res[0], 60)
+
+    def test_auto_ret_type(self):
+        data = [
+            (datetime(2019, 1, 1), datetime(2020, 1, 1), datetime),
+            (1552252218.0, 1591823311.0, float),
+        ]
+        for start, stop, rtype in data:
+            ret = list(croniter_range(start, stop, "0 0 * * *"))
+            self.assertIsInstance(ret[0], rtype)
+
+    def test_input_type_exceptions(self):
+        dt_start1 = datetime(2019, 1, 1)
+        dt_stop1 = datetime(2020, 1, 1)
+        f_start1 = 1552252218.0
+        f_stop1 = 1591823311.0
+        # Mix start/stop types
+        with self.assertRaises(TypeError):
+            list(croniter_range(dt_start1, f_stop1, "0 * * * *"), ret_type=datetime)
+        with self.assertRaises(TypeError):
+            list(croniter_range(f_start1, dt_stop1, "0 * * * *"))
+
+    def test_timezone_dst(self):
+        """ Test across DST transition, which technially is a timzone change. """
+        tz = pytz.timezone("US/Eastern")
+        start = tz.localize(datetime(2020, 10, 30))
+        stop =  tz.localize(datetime(2020, 11, 10))
+        res = list(croniter_range(start, stop, '0 0 * * *'))
+        self.assertNotEqual(res[0].tzinfo, res[-1].tzinfo)
+        self.assertEqual(len(res), 12)
+
+    def test_extra_hour_day_prio(self):
+        def datetime_tz(*args, **kw):
+            """ Defined this in another branch.  single-use-version """
+            tzinfo = kw.pop("tzinfo")
+            return tzinfo.localize(datetime(*args))
+        tz = pytz.timezone("US/Eastern")
+        cron = "0 3 * * *"
+        start = datetime_tz(2020, 3, 7, tzinfo=tz)
+        end = datetime_tz(2020, 3, 11, tzinfo=tz)
+        ret = [ i.isoformat() for i in croniter_range(start, end, cron) ]
+        self.assertEqual(ret, [
+            "2020-03-07T03:00:00-05:00",
+            "2020-03-08T03:00:00-04:00",
+            "2020-03-09T03:00:00-04:00",
+            "2020-03-10T03:00:00-04:00"])
+
+    def test_issue_142_dow(self):
+        ret = []
+        for i in range(1, 31):
+            ret.append((i,
+                croniter('35 * 0-l/8 * *', datetime(2020, 1, i),
+                         ret_type=datetime).get_next())
+            )
+            i += 1
+        self.assertEqual(
+            ret,
+            [(1, datetime(2020, 1, 1, 0, 35)),
+             (2, datetime(2020, 1, 8, 0, 35)),
+             (3, datetime(2020, 1, 8, 0, 35)),
+             (4, datetime(2020, 1, 8, 0, 35)),
+             (5, datetime(2020, 1, 8, 0, 35)),
+             (6, datetime(2020, 1, 8, 0, 35)),
+             (7, datetime(2020, 1, 8, 0, 35)),
+             (8, datetime(2020, 1, 8, 0, 35)),
+             (9, datetime(2020, 1, 16, 0, 35)),
+             (10, datetime(2020, 1, 16, 0, 35)),
+             (11, datetime(2020, 1, 16, 0, 35)),
+             (12, datetime(2020, 1, 16, 0, 35)),
+             (13, datetime(2020, 1, 16, 0, 35)),
+             (14, datetime(2020, 1, 16, 0, 35)),
+             (15, datetime(2020, 1, 16, 0, 35)),
+             (16, datetime(2020, 1, 16, 0, 35)),
+             (17, datetime(2020, 1, 24, 0, 35)),
+             (18, datetime(2020, 1, 24, 0, 35)),
+             (19, datetime(2020, 1, 24, 0, 35)),
+             (20, datetime(2020, 1, 24, 0, 35)),
+             (21, datetime(2020, 1, 24, 0, 35)),
+             (22, datetime(2020, 1, 24, 0, 35)),
+             (23, datetime(2020, 1, 24, 0, 35)),
+             (24, datetime(2020, 1, 24, 0, 35)),
+             (25, datetime(2020, 2, 1, 0, 35)),
+             (26, datetime(2020, 2, 1, 0, 35)),
+             (27, datetime(2020, 2, 1, 0, 35)),
+             (28, datetime(2020, 2, 1, 0, 35)),
+             (29, datetime(2020, 2, 1, 0, 35)),
+             (30, datetime(2020, 2, 1, 0, 35))])
+
+    def test_issue145_getnext(self):
+        # Example of quarterly event cron schedule
+        start = datetime(2020, 9, 24)
+        cron = "0 13 8 1,4,7,10 wed"
+        with self.assertRaises(CroniterBadDateError):
+            it = croniter(cron, start, day_or=False, max_years_between_matches=1)
+            it.get_next()
+        # New functionality (0.3.35) allowing croniter to find spare matches of cron patterns across multiple years
+        it = croniter(cron, start, day_or=False, max_years_between_matches=5)
+        self.assertEqual(it.get_next(datetime), datetime(2025, 1, 8, 13))
+
+    def test_issue145_range(self):
+        cron = "0 13 8 1,4,7,10 wed"
+        matches = list(croniter_range(datetime(2020, 1, 1), datetime(2020, 12, 31), cron, day_or=False))
+        self.assertEqual(len(matches), 3)
+        self.assertEqual(matches[0], datetime(2020, 1, 8, 13))
+        self.assertEqual(matches[1], datetime(2020, 4, 8, 13))
+        self.assertEqual(matches[2], datetime(2020, 7, 8, 13))
+
+        # No matches within this range; therefore expect empty list
+        matches = list(croniter_range(datetime(2020, 9, 30), datetime(2020, 10, 30), cron, day_or=False))
+        self.assertEqual(len(matches), 0)
+
+    def test_explicit_year_forward(self):
+        start = datetime(2020, 9, 24)
+        cron = "0 13 8 1,4,7,10 wed"
+
+        # Expect exception because no explict range was provided.  Therefore, the caller should be made aware that an implicit limit was hit.
+        ccron = croniter(cron, start, day_or=False)
+        ccron._max_years_between_matches = 1
+        iterable = ccron.all_next()
+        with self.assertRaises(CroniterBadDateError):
+            next(iterable)
+
+        iterable = croniter(cron, start, day_or=False, max_years_between_matches=5).all_next(datetime)
+        n = next(iterable)
+        self.assertEqual(n, datetime(2025, 1, 8, 13))
+
+        # If the explictly given lookahead isn't enough to reach the next date, that's fine.  The caller specified the maximum gap, so no just stop iteration
+        iterable = croniter(cron, start, day_or=False, max_years_between_matches=2).all_next(datetime)
+        with self.assertRaises(StopIteration):
+            next(iterable)
+
+    def test_explicit_year_reverse(self):
+        start = datetime(2025, 1, 1)
+        cron = "0 13 8 1,4,7,10 wed"
+
+        ccron = croniter(cron, start, day_or=False)
+        ccron._max_years_between_matches = 1
+        iterable = ccron.all_prev()
+        with self.assertRaises(CroniterBadDateError):
+            next(iterable)
+
+        iterable = croniter(cron, start, day_or=False, max_years_between_matches=5).all_prev(datetime)
+        n = next(iterable)
+        self.assertEqual(n, datetime(2020, 7, 8, 13))
+
+        iterable = croniter(cron, start, day_or=False, max_years_between_matches=2).all_prev()
+        with self.assertRaises(StopIteration):
+            next(iterable)
+
+    def test_isssue151(self):
+        """."""
+        self.assertTrue(
+            croniter.match("* * * * *", datetime(2019, 1, 14, 11, 0, 59, 999999)))
+
+
 if __name__ == '__main__':
     unittest.main()
